/******************************************************************************
 *  Datei ChkArch2.C
 *  Enthlt die Funktion ChkAuft() fr das Programm GalAsm, den Logic-
 *  Compiler / -Assembler zur Erstellung einer JEDEC-Datei
 *  fr die GAL-Programmierung.
 *
 *  Version               2.0
 *  Erstellt am           13.08.1990
 *  Letzte nderung am    26.05.1991
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien: GalAsm.H      Makros und Def. der Var.'typen
 *                           GalGlVar.H    Deklaration der globalen Variablen
 *                           HilfFktn.H
 *
 *  Bentigte C-Module:      GalGlVar.C    Definition der globalen Variablen
 *                           HilfFktn.C
 *
 *****************************************************************************/

#include <stdio.h>

#include "galasm.h"
#include "galglvar.h"
#include "hilffktn.h"


/*--------------------------------------------------------------------------
 |  Funktion ChkArch()
 |  berprft die Vertrglichkeit zwischen Architekturkontrollbits, den
 |  Rckfhrungen der Ausgangssingale, der Tri-State-Freigabebedingungen
 |  sowie Ausgangsregister der progammierten Ein- und Ausgnge, sowie den
 |  Signalen Clk (Pin 1) und /OE (Pin 11 bzw. 13) und den Pins 14 und 23 bei
 |  Gals des Typs 20V8, soweit sie in der Logik-Matrix bentigt werden.
 |  Fehlercodes:
 */
#define CHKARCH_ERR   30000L /* Basis-Fehlernummer der Fkt. ChkArch()        */
#define OE_CLK_N_EING   300L /* OE, Clk nicht Eingang                        */
#define OLMC_N_RCKK     400L /* i.OLMC: Nicht rckfhrbar                    */
#define KEIN_TS         500L /* i.OLMC: Kein Tri-State mglich               */
#define KEIN_REG        600L /* i.OLMC: Kein registered output mglich       */
#define KEINE_PTF       700L /* i.OLMC: Keine Produkttermfreigabe mglich    */
#define KEINE_OE_FREIG  800L /* i.OLMC: Kein OE-TS mglich                   */
#define KEIN_TSPT_FREI  900L /* i.OLMC: Kein Produktterm fr TS-Freigabe frei*/
#define KEIN_AUSG      1000L /* i.OLMC: Kein Ausgang                         */
#define KEIN_EING      1100L /* i.OLMC: Kein Eingang mglich                 */
 /*
 |  Fr i. wird ein Wert von 0 bis 7 auf die Fehlernummer aufaddiert.
 |  0 entspricht OLMC Nr. 8 (beim OE-Pin) 7 entspricht OLMC Nr. 1 (bei Vcc).
 *--------------------------------------------------------------------------*/

long ChkArch()
{
 register int i;
  int RegFlag;
  int AlleRueckgk;
  int PTFFlag;

  gotoxy(0,MSG_ZEILE);
  end_line();
#ifdef GALASM_DEBUG  
  printf("Kontrolle, ob erforderliche Rckfhrungen mglich...\n");
#endif
  RegFlag = 0;
  AlleRueckgk = 1;
  PTFFlag = 0;

  /*=========================================================================*
   *         die vom Parser nicht gesetzten Flags setzen                     *
   *=========================================================================*/
  for(i=0;i<AUSGAENGE;i++) {
    if(! Ausgaenge[i].Genutzt) { /* unbenutzte Ausgnge zu Eingngen machen */
      Ausgaenge[i].Hochohmig = 1;
    }
    if(Ausgaenge[i].Reg) {        /* alle Registered-Ausgnge haben OE-Freig */
      Ausgaenge[i].OEFreig = 1;
      RegFlag = 1;
    }
    if((  Ausgaenge[i].Ausg) &&                  /* Ausgnge ohne     */
       (! Ausgaenge[i].ProdFreig)  &&            /* Freigabebedingung */
       (! Ausgaenge[i].OEFreig)        )   {     /* sind niederohmig  */
          Ausgaenge[i].Niederohmig = 1;
    }
    if((  Ausgaenge[i].Genutzt) &&               /* Eingnge sind     */
       (! Ausgaenge[i].Ausg)     ) {             /* hochohmig zu beschalten */
          Ausgaenge[i].Hochohmig = 1;
    }
    if(!(Ausgaenge[i].Rueckkop))                   /* Ausgang nicht rckgek. */
      AlleRueckgk = 0;
    if(Ausgaenge[i].ProdFreig)
      PTFFlag = 1;
  } /* for */
  if(RegFlag) { /* Freigabe durch /OE bei komb. Ausgngen nur m. Prod.term*/
    for(i=0;i<AUSGAENGE;i++) {
      if((Ausgaenge[i].OEFreig) &&
         (!(Ausgaenge[i].Reg))      ) {
           OEinMatrix = 1;
      }
    }
  }
  /**************************************************************************
   *     Architekturkontrollbits einstellen, soweit noch nicht angegeben    *
   **************************************************************************/

  /*-------------- es liegen Registered Outputs vor ---------------------*/
  if(RegFlag) {
    if(SynBit < 0) SynBit = 0;
    if(AC0Bit < 0) AC0Bit = 1;
    for(i=0;i<AUSGAENGE;i++) {
      if((Ausgaenge[i].OEFreig) &&
         (!(Ausgaenge[i].Reg))      ) {
           return(CHKARCH_ERR + KEINE_OE_FREIG + (long)i);
      }  /* if */
    } /* for */
    if((OEinMatrix) || (ClkinMatrix))  {
      return(CHKARCH_ERR + KEINE_OE_FREIG + OE_CLK_N_EING);
    }
    for(i=0;i<AUSGAENGE;i++) {
      if(Ausgaenge[i].Reg)  {
        if(AC1Bits[i] <0) AC1Bits[i] = 0;
      }  /* if */
      else {
        if(AC1Bits[i] < 0) AC1Bits[i] = 1;
      }
    } /* for */
  }
  /*-------------- es liegen keine Registered Outputs vor -------------------*/
  else {   /* 1. else */
    for(i=0;i<AUSGAENGE;i++) {
      if((Ausgaenge[i].OEFreig) &&
         (!(Ausgaenge[i].Reg))      ) {
               OEinMatrix = 1;
      }  /* if */
    } /* for */
    if( (!(OEinMatrix))   &&
        (!(ClkinMatrix))  &&
        (AlleRueckgk)       )           {
          if(SynBit < 0) SynBit = 0;
          if(AC0Bit < 0) AC0Bit = 1;
          for(i=0;i<AUSGAENGE;i++) {
            if(AC1Bits[i] < 0) AC1Bits[i] = 1;
          } /* for */
    }
    else {   /* 2. else */
      if(SynBit < 0) SynBit = 1;
      if(Ausgaenge[AUSGAENGE/2].Rueckkop) {
        if(AC0Bit < 0) AC0Bit = 1;
        if(AC1Bits[AUSGAENGE/2] <0) AC1Bits[AUSGAENGE/2] = 1;
        if(Ausgaenge[0].Rueckkop) return(CHKARCH_ERR + OLMC_N_RCKK + 0L);
        if(Ausgaenge[AUSGAENGE-1].Rueckkop)
          return(CHKARCH_ERR + OLMC_N_RCKK + (long)(AUSGAENGE-1));
      }
      else { /* 3. else */
        if(Ausgaenge[(AUSGAENGE/2) - 1].Rueckkop) {
          if(AC0Bit < 0) AC0Bit = 1;
          if(AC1Bits[(AUSGAENGE/2) - 1] < 0) AC1Bits[(AUSGAENGE/2) - 1] = 1;
          if(Ausgaenge[0].Rueckkop) return(CHKARCH_ERR + OLMC_N_RCKK + 0L);
          if(Ausgaenge[AUSGAENGE-1].Rueckkop)
            return(CHKARCH_ERR + OLMC_N_RCKK + (long)(AUSGAENGE-1));
        }
        else { /* 4. else */
          if(Ausgaenge[AUSGAENGE - 1].Rueckkop) {
            if(AC0Bit < 0) AC0Bit = 0;
            if(AC1Bits[AUSGAENGE - 1] < 0) AC1Bits[AUSGAENGE - 1] = 1;
            if(PTFFlag) return(CHKARCH_ERR+OLMC_N_RCKK + (long)(AUSGAENGE-1));
          }
          else { /* 5. else */
            if(Ausgaenge[0].Rueckkop) {
              if(AC0Bit < 0) AC0Bit = 0;
              if(AC1Bits[0] < 0) AC1Bits[0] = 1;
              if(PTFFlag) return(CHKARCH_ERR + OLMC_N_RCKK + 0L);
            }
            else { /* 6. else */
              for(i=0;i<AUSGAENGE;i++) {
                if( ( Ausgaenge[i].ProdFreig) ||
                    ((Ausgaenge[i].Rueckkop) && (Ausgaenge[i].Ausg))  ) {
                  if(AC0Bit < 0) AC0Bit = 1;
                  if(AC1Bits[i] < 0) AC1Bits[i] = 1;
                }  /* if */
              } /* for */
              if(AC0Bit < 0) AC0Bit = 0;
            } /* 6. else */
          } /* 5. else */
        } /* 4. else */
      } /* 3. else */
      if(AC0Bit == 1) {
        for(i=0;i<AUSGAENGE;i++) {
          if(AC1Bits[i] < 0) {
            if(Ausgaenge[i].Genutzt) AC1Bits[i] = 1;
            else {
             AC1Bits[i] = 0;
            }
          }  /* if */
        } /* for */
      }
      else {  /* ! AC0Bit */
        for(i=0;i<AUSGAENGE;i++) {
          if(AC1Bits[i] < 0) {
            if(!(Ausgaenge[i].Genutzt)) AC1Bits[i] = 1;
            else {
              if(Ausgaenge[i].Ausg) AC1Bits[i] = 0;
              else AC1Bits[i] = 1;
            }
          }  /* if */
        } /* for */
      } /* !AC0Bit */
    }   /* 2. else */
  } /*-1.else---- es liegen keine Registered Outputs vor -------------------*/

  /*--------- Kontrolle des Syn-Bits ---------------------------------------*/
  if(((OEinMatrix) || (ClkinMatrix)) && (! SynBit)) {
    /* Signale OE und Clk knnen nicht als Eingnge genutzt werden */
    return(CHKARCH_ERR + OE_CLK_N_EING);
  }
  /*---- Kontrolle des Syn-Bits durch Rckkopplung der ueren Ausgnge -----*/
  if((SynBit) && (AC0Bit)) {
    if(Ausgaenge[0].Rueckkop)  {
          /* Ausgaenge[0] nicht rckfhrbar */
          return(CHKARCH_ERR + OLMC_N_RCKK + 0L);
    }
    if(Ausgaenge[AUSGAENGE-1].Rueckkop) {
          /* Ausgaenge[AUSGAENGE-1] kann nicht zurckgefhrt werden */
          return(CHKARCH_ERR + OLMC_N_RCKK + (long)AUSGAENGE - 1L);
    }
  }
  /*---------- Kontrolle des AC0-Bits durch Tri-State-Ausgnge -------------*/
  for(i=1;i<(AUSGAENGE-1);i++) {
    if(((Ausgaenge[i].ProdFreig)||(Ausgaenge[i].OEFreig)) &&
        ( ! AC0Bit))     {
          /* Ausgaenge[i]: Kein Tri-State mglich */
          return(CHKARCH_ERR + KEIN_TS + (long)i);
    }
  }
  /*----- Kontrolle des AC0-Bits ber Rckkopplung der inneren OLMCs  -----*/
  if(! AC0Bit) {
    if(Ausgaenge[AUSGAENGE/2].Rueckkop)  {
      /* Ausgaenge[AUSGAENGE/2] kann nicht zurckgefhrt werden */
      return(CHKARCH_ERR + OLMC_N_RCKK + (long)(AUSGAENGE/2));
    }
    if(Ausgaenge[(AUSGAENGE/2)-1].Rueckkop)  {
      /* Ausgaenge[(AUSGAENGE/2)-1] kann nicht zurckgefhrt werden */
          return(CHKARCH_ERR + OLMC_N_RCKK + (long)((AUSGAENGE/2)-1));
    }
  }
  /*----- Kontrolle der AC0- AC1(0)- und AC1(7)-Bits ber --------
    --------------------- Rckkopplung der ueren OLMCs  -------------------*/
  if((SynBit) && (! AC0Bit)) {
    if((Ausgaenge[0].Rueckkop)&&(! AC1Bits[0])) {
      /* Ausgaenge[0] kann nicht zurckgefhrt werden */
      return(CHKARCH_ERR + OLMC_N_RCKK + 0L);
    }
    if((Ausgaenge[AUSGAENGE-1].Rueckkop)&&(! AC1Bits[7]))   {
      /* Ausgaenge[AUSGAENGE-1] kann nicht zurckgefhrt werden */
      return(CHKARCH_ERR + OLMC_N_RCKK + (long)(AUSGAENGE - 1));
    }
  }
  /*--------- Kontrolle des Output-Mux durch Registered-Ausgnge ------------*/
  for(i=0;i<AUSGAENGE;i++) {
    if((Ausgaenge[i].Reg) && (! AC0Bit) && (AC1Bits[i])) {
      /* Ausgaenge[i]: Kein registered output mglich */
      return(CHKARCH_ERR + KEIN_REG + (long)i);
    }
  }
  /*-- Kontrolle der Tri-State-Mux. Ggf. nderung d. Freigabeprod.'termes --*/
  for(i=0;i<AUSGAENGE;i++) {
    if((Ausgaenge[i].ProdFreig) &&
       ((Ausgaenge[i].Reg) || (! AC1Bits[i])))  {
          /* Ausgaenge[i]: Keine Produkttermfreigabe mglich */
          return(CHKARCH_ERR + KEINE_PTF + (long)i);
    }
    if(Ausgaenge[i].OEFreig) {
      if(AC1Bits[i]) {      /* wenn AC1(n) gesetzt, dann OE ber Logikmatrix */
        if(! SynBit) {
          /* Ausgaenge[i]: Kein OE-Tri-State mglich, da OE nicht in Matrix */
          return(CHKARCH_ERR + KEINE_OE_FREIG + (long)i);
        }
        if(ProdTermFrei(i)) {
          /* Ausgaenge[i]: Kein Produktterm fr TS-Freigabe frei */
          return(CHKARCH_ERR + KEIN_TSPT_FREI + (long)i);
        }
        LogikMatrix[i][0][0] = -OEPin;      /* inv. OE-Signal eintragen */
        Ausgaenge[i].ProdFreig = 1;         /* Flag setzen */
      } /* if(AC1Bits) */
    }  /* if(OEFreig) */
    if(Ausgaenge[i].Niederohmig) {          /* kein TS-Ausgang */
      if(AC0Bit) {
        if(! AC1Bits[i]) {
          /* Ausgaenge[i]: Ausgang nur ber Produkttermfreigabe mgl.*/
          return(CHKARCH_ERR + KEIN_AUSG + (long)i);
        }
        if(ProdTermFrei(i))  {        /* alle Verbindungen offen lassen */
          /* Ausgaenge[i]: Kein Produktterm fr TS-Freigabe frei */
          return(CHKARCH_ERR + KEIN_TSPT_FREI + (long)i);
        }
        LogikMatrix[i][0][0] =            /* Markieren, da alle Verbin- */
          NICHTS_BRENNEN;                 /* dungen offenzulassen sind */
      }
      else {
        if(AC1Bits[i]) {
          /* Ausgaenge[i]: Kein Ausgang mglich */
          return(CHKARCH_ERR + KEIN_AUSG + (long)i);
        }
      }
    }
    if(Ausgaenge[i].Hochohmig) {      /* Eingang */
      if(AC0Bit) {
        if( (! AC1Bits[i])         &&
          ( Ausgaenge[i].Genutzt)  ) {
          /* Ausgaenge[i]: Kein Eingang mglich */
          return(CHKARCH_ERR + KEIN_EING + (long)i);
        }
        if(ProdTermFrei(i))    {         /* alle Verbindungen herstellen   */
          /* Ausgaenge[i]: Kein Produktterm fr TS-Freigabe frei */
          return(CHKARCH_ERR + KEIN_TSPT_FREI + (long)i);
        }
        LogikMatrix[i][0][0] =            /* Markieren, da alle Verbin- */
          ALLES_BRENNEN;                  /* dungen herzustellen sind */
      }
      else {
        if( ! AC1Bits[i]) {
          /* Ausgaenge[i]: Ausgang nur ber Produkttermfreigabe mgl.*/
          return(CHKARCH_ERR + KEIN_AUSG + (long)i);
        }
      }
    }
  }
  /*-- Kontrolle der Feetback-Mux der inneren OLMCs -----*/
  if(! AC0Bit) {  /* keine Rckkopplg. der inneren OLMCs mgl. */
    if(Ausgaenge[AUSGAENGE/2].Rueckkop)  {
      /* Ausgaenge[AUSGAENGE/2] kann nicht zurckgefhrt werden */
      return(CHKARCH_ERR + OLMC_N_RCKK + (long)(AUSGAENGE/2));
    }
    if(Ausgaenge[(AUSGAENGE/2)-1].Rueckkop)  {
          /* Ausgaenge[(AUSGAENGE/2)-1] kann nicht zurckgefhrt werden */
          return(CHKARCH_ERR + OLMC_N_RCKK + (long)(AUSGAENGE/2-1));
    }
  }
  return(KEIN_FEHLER);
}

