/******************************************************************************
 *  Datei FuseMap.C
 *  Enthlt die Funktion FuseMap() fr das Programm GalAsm, den Logic-
 *  Compiler / -Assembler zur Erstellung einer JEDEC-Datei
 *  fr die GAL-Programmierung.
 *
 *  Version               1.2
 *  Erstellt am           16.08.1990
 *  Letzte nderung am    3.1.1991
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien: GalAsm.H      Makros und Def. der Var.'typen
 *                           GalGlVar.H    Deklaration der globalen Variablen
 *
 *  Bentigte C-Module:      GalGlVar.C    Definition der globalen Variablen
 *
 *****************************************************************************/

#include <stdio.h>
#include <math.h>

#include "galasm.h"
#include "galglvar.h"


/*------------------------------------------------------------------------
 |  Funktion FuseMap()
 |  Mit Hilfe der Zuordnugstabelle Pinnummer <-> Matrixzeile und dem vom
 |  Parser erzeugten Feld die Fusemap berechnen. Gleichzeitig Porduktterm-
 |  freigabebits der nichtbentigten Produktterme lschen. Fusemap direkt
 |  in die Zieldatei eintragen.
 |  Produkttermfreigabebits und OLMC-Konfigurationsbits eintragen.
 |  Fehlercodes:
 */
#define  FUSEMAP_ERR       50000L      /* Basis-Fehlernummer von FuseMap() */
#define  PARSER_TAB_FEHLER   300L      /* Fehlerhafte Parser-Tabelle */
 /*
 *------------------------------------------------------------------------*/

long FuseMap()
{
  char MatrixZeile[2* MAX_ANZ_SPALTEN];    /* Zwischenspeicher f. eine Zeile */
  register int  LfdAusgNr;       /* Laufvariable ber alle Ausgnge */
  register int  LfdProdTermNr;   /* Laufvariable ber alle Produktterme */
  register int  LfdSpaltenNr;    /* Laufvariable ber Matrixspalten */
  register int  i,k;
  unsigned char Maske;
  int AktPinNr;

  for(LfdAusgNr=AUSGAENGE-1;LfdAusgNr >= 0;LfdAusgNr--) { /* ber alle Ausg. */
    gotoxy(0,MSG_ZEILE);
    end_line();
#ifdef GALASM_DEBUG
    printf("Anlegen der JEDEC-Datei, Ausgang %d\r",LfdAusgNr+1);
#endif
    if(! fprintf(ZielFP,"\n*N IO-Pin-Nr: %d, Signal: ",
                        Ausgaenge[LfdAusgNr].PinNummer))
      return(FUSEMAP_ERR + SCHREIBFEHLER);
    if(Ausgaenge[LfdAusgNr].LowAktiv) {  /* Signal ist low-aktiv */
      if(! fprintf(ZielFP,"/")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    if(! fprintf(ZielFP,"%s ",Ausgaenge[LfdAusgNr].SigName))
      return(FUSEMAP_ERR + SCHREIBFEHLER);
    if(Ausgaenge[LfdAusgNr].Ausg) {  /* Signal ist Ausgang */
      if(! fprintf(ZielFP,"(Ausgang)\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    else {                           /* Signal ist Eingang */
      if(! fprintf(ZielFP,"(Eingang)\n"))
        return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    for(LfdProdTermNr=0;LfdProdTermNr<ANZ_PROD;LfdProdTermNr++) {
      ProdTermFreig[LfdAusgNr][LfdProdTermNr] = 0;   /* Ptf-Bit lschen */
      AktPinNr = LogikMatrix[LfdAusgNr][LfdProdTermNr][0];
      if(AktPinNr == ALLES_BRENNEN) {    /* Ganze Zeile brennen */
        ProdTermFreig[LfdAusgNr][LfdProdTermNr] = 1;   /* Ptf-Bit setzen */
        for( LfdSpaltenNr=0;
             LfdSpaltenNr < (2 * AnzSpalten);
             MatrixZeile[LfdSpaltenNr++] = 0);   /* 0 = Zelle brennen */
      }
      else {
        if(AktPinNr == NICHTS_BRENNEN) {    /* Ganze Zeile offenlassen */
          ProdTermFreig[LfdAusgNr][LfdProdTermNr] = 1;   /* Ptf-Bit setzen */
          for( LfdSpaltenNr=0;
            LfdSpaltenNr < (2 * AnzSpalten);
            MatrixZeile[LfdSpaltenNr++] = 1);   /* 0 = Zelle brennen */
        }
        else {                 /* Zeile entsprechend Logikmatrix brennen */
          for( LfdSpaltenNr=0;
               LfdSpaltenNr < (2 * AnzSpalten);    /* Zeile zuerst lschen */
            MatrixZeile[LfdSpaltenNr++] = 1);  /* 1 = Zelle ungebrannt */
            for(LfdSpaltenNr=0; LfdSpaltenNr < AnzSpalten;LfdSpaltenNr++) {
            AktPinNr = LogikMatrix[LfdAusgNr][LfdProdTermNr][LfdSpaltenNr];
            if(AktPinNr) {
              if((InvPinInSp[(abs(AktPinNr))-1] == -1) ||
                 (PosPinInSp[(abs(AktPinNr))-1] == -1))       {
                gotoxy(0,WARN_ZEILE);
                end_line();
                fprintf(stderr, "*** Warnung:Signal an PinNr %d nicht auf Matrix gefhrt ***\n",
                        abs(AktPinNr));
              }
              ProdTermFreig[LfdAusgNr][LfdProdTermNr] = 1; /* Ptf-Bit setzen */
              if((AktPinNr < 0) &&        /* Signal invers */
                (AktPinNr > -AnzPins)) {  /* Signal gltig */
                MatrixZeile[(InvPinInSp[(-AktPinNr)-1])] = 0; /* Bit lschen */
              }
              else {
                if((AktPinNr > 0) &&        /* Signal positiv */
                  (AktPinNr < AnzPins)) {   /* Signal gltig */
                  MatrixZeile[(PosPinInSp[AktPinNr-1])] = 0; /* Bit lschen */
                }
                else return(FUSEMAP_ERR + PARSER_TAB_FEHLER);
              }
            } /* if(AktPinNr) */
          } /* for(LfdSpaltenNr) */
        } /* else */
      } /* else */
      if(ProdTermFreig[LfdAusgNr][LfdProdTermNr]) {     /* Ptf-Bit gesetzt ? */
        if(! fprintf(ZielFP,"*L%04d  ",
            ((((AUSGAENGE-1)-LfdAusgNr) * ANZ_PROD  ) +
               LfdProdTermNr) * 2 * AnzSpalten)        )
          return(FUSEMAP_ERR + SCHREIBFEHLER);
        for(LfdSpaltenNr=0; LfdSpaltenNr < (2 * AnzSpalten);LfdSpaltenNr++) {
          if(!(LfdSpaltenNr % 4))
            if(! fprintf(ZielFP," "))  /* formatierte Ausgabe ! */
              return(FUSEMAP_ERR + SCHREIBFEHLER);
          if(! fprintf(ZielFP,"%d",MatrixZeile[LfdSpaltenNr]))
            return(FUSEMAP_ERR + SCHREIBFEHLER);
        } /* for(LfdSpaltenNr) */
        if(! fprintf(ZielFP,"\n"))
          return(FUSEMAP_ERR + SCHREIBFEHLER);
      }
    }  /* for(LfdProdTermNr) */
  }  /* for(LfdAusgNr)  */
#ifdef GALASM_DEBUG
  printf("\n");
#endif
  /*----------------------------------- Xor-Bit eintragen -------------------*/
  if(! fprintf(ZielFP,"\n*N XOR-Bits\n*L%04d  ",AdrXORBits))
    return(FUSEMAP_ERR + SCHREIBFEHLER);
  for(LfdAusgNr = AUSGAENGE-1; LfdAusgNr >=0; LfdAusgNr--) {
    if(Ausgaenge[LfdAusgNr].Negieren) {   /* Ausg'sig. invertieren ? */
     if(! fprintf(ZielFP,"0"))          /* ja   -> XorBit = 0 */
       return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    else {
     if(! fprintf(ZielFP,"1"))             /* nein -> XorBit = 1 */
       return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
  }
  if(! fprintf(ZielFP,"\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
  /*--------------------------------User-Signatur eintragen ---------------*/
  if(! fprintf(ZielFP,"\n*N 64 Bits User-Signatur (Inhalt: %s)\n*L%04d ",
                       UserElSig,AdrSigBits))
    return(FUSEMAP_ERR + SCHREIBFEHLER);
  for(i=0;i<UES_LEN ;i++) {
    if(! fprintf(ZielFP," "))
      return(FUSEMAP_ERR + SCHREIBFEHLER);
    Maske = (unsigned char)(1 << 7);
    for(k=0; k < 8; k++) {
      if(Maske & UserElSig[i]) {
        if(! fprintf(ZielFP,"1")) return(FUSEMAP_ERR + SCHREIBFEHLER);
      } else {
        if(! fprintf(ZielFP,"0")) return(FUSEMAP_ERR + SCHREIBFEHLER);
      }
      if(k<8) Maske >>= 1;
    } /* for k */
  } /* for i */
  /*---------------------------------- AC1-Bits eintragen -------------------*/
  if(! fprintf(ZielFP,"\n\n*N AC1-Bits\n*L%04d  ",
                       AdrAC1Bits))
    return(FUSEMAP_ERR + SCHREIBFEHLER);
  for(LfdAusgNr = AUSGAENGE-1; LfdAusgNr >=0; LfdAusgNr--) {
    if(AC1Bits[LfdAusgNr]) {        /* AC1-Bit gesetzt ? */
     if(! fprintf(ZielFP,"1")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    else {
     if(! fprintf(ZielFP,"0")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
  }
  if(! fprintf(ZielFP,"\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
  /*--------------------Produktterm-Freigabe-Bits eintragen -----------------*/
  for(LfdAusgNr = AUSGAENGE-1;LfdAusgNr >= 0;LfdAusgNr--) {
    if(! fprintf(ZielFP,"\n*N Produkttermfreigabe IO-Pin-Nr: %d\n*L%04d  ",
            Ausgaenge[LfdAusgNr].PinNummer,
            AdrPTFBits +                  /* Adr. des ersten PTF-Bytes */
            8*((AUSGAENGE-1)-LfdAusgNr) ))
      return(FUSEMAP_ERR +SCHREIBFEHLER);
    for(LfdProdTermNr=0;LfdProdTermNr<ANZ_PROD;LfdProdTermNr++) {
      if(ProdTermFreig[LfdAusgNr][LfdProdTermNr]) {  /* Ptf-Bit gesetzt ? */
        if(! fprintf(ZielFP,"1"))
          return(FUSEMAP_ERR + SCHREIBFEHLER);
      }
      else {
        if(! fprintf(ZielFP,"0")) return(FUSEMAP_ERR + SCHREIBFEHLER);
      }
    }
  }
  /*------------------ Warnung, wenn SYN != AC0 -----------------------------*/
  if(SynBit != AC0Bit) {
    fprintf(stderr,"\n****** WARNUNG: SYN und AC0 unterscheiden sich ****** \n");
    fprintf(stderr,"\t\tVorsicht bei A-GALs und MAXON MGP 2.0 (ggf. Parameter -%c)\n\n",
    		FIX_MAXON);
  }
  /*------------------------------------- Syn-Bit eintragen -----------------*/
  if(FixMaxon == 0) {
    if(! fprintf(ZielFP,"\n\n*N Syn-Bit\n*L%04d  ",AdrSynBit))
      return(FUSEMAP_ERR + SCHREIBFEHLER);
    if(SynBit) {
      if(! fprintf(ZielFP,"1\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    else {
      if(! fprintf(ZielFP,"0\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
  } else {
    /*----------------- Korrektur fuer A-GAL und MGP Ver. 2.0 -----------------*/
    if(! fprintf(ZielFP,"\n\n*N Vorsicht: Syn-Bit und AC0-Bit vertauscht!"))
      return(FUSEMAP_ERR + SCHREIBFEHLER);  
    if(! fprintf(ZielFP,"\n*N Syn-Bit\n*L%04d  ",AdrSynBit))
      return(FUSEMAP_ERR + SCHREIBFEHLER);
    if(AC0Bit) {
      if(! fprintf(ZielFP,"1\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    else {
      if(! fprintf(ZielFP,"0\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
  }
  /*------------------------------------- AC01-Bit eintragen -----------------*/
  if(FixMaxon == 0) {  
    if(! fprintf(ZielFP,"\n*N AC0-Bit\n*L%04d  ",AdrAC0Bit))
      return(FUSEMAP_ERR + SCHREIBFEHLER);
    if(AC0Bit) {
      if(! fprintf(ZielFP,"1\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    else {
      if(! fprintf(ZielFP,"0\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    } 
  } else {
    /*----------------- Korrektur fuer A-GAL und MGP Ver. 2.0 -----------------*/  
    if(! fprintf(ZielFP,"\n\n*N Vorsicht: Syn-Bit und AC0-Bit vertauscht!"))
      return(FUSEMAP_ERR + SCHREIBFEHLER);    
    if(! fprintf(ZielFP,"\n*N AC0-Bit\n*L%04d  ",AdrAC0Bit))
      return(FUSEMAP_ERR + SCHREIBFEHLER);
    if(SynBit) {
      if(! fprintf(ZielFP,"1\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }
    else {
      if(! fprintf(ZielFP,"0\n")) return(FUSEMAP_ERR + SCHREIBFEHLER);
    }   
  }
  /*--------------------------------------------------------------------------*/
  
  if(! fprintf(ZielFP,"*%c0000",ETX)) return(FUSEMAP_ERR + SCHREIBFEHLER);
  return(KEIN_FEHLER);
}
