/******************************************************************************
 *  Datei GalAsm.C, enthlt den Hauptteil (die Funktion main()) fr das
 *  Programm
 *
 *           GGGGG         ll     AA
 *          GG    G         l    AA A
 *          GG       aaaa   l   AA   A   sss  mmm mm
 *          GG GGGG      a  l  AA     A s     m  m  m
 *          GG    G  aaaaa  l  AAAAAAAA  sss  m  m  m
 *          GG    G a    a  l  AA     A     s m  m  m
 *           GGGGG   aaaaa lll AA     A  sss  m  m  m
 *
 *  den Logic-Compiler / -Assembler zur Erstellung einer JEDEC-Datei fr die
 *  GAL-Programmierung.
 *
 *
 *  Versionsnummer:
 */
char *VersionNr = "1.5 (Linux)";
/*
 *  Erstellt am         05.08.1990
 *  nderungen:		17.Mar.1994: Portierung auf Linux.
 */
char *ErstDatum = "26.Dec.1995";
/*
 *  Verwendeter Compiler:  gcc 
 *
 ***************************************************************************
 *                                                                         *
 *             FFFFFF                   SSSS  ii  ll  k                  *
 *             FF                       SS    S      l  k  k               *
 *             FF         r rr        SS      ii   l  kkk     eeee       *
 *             FFFF       rr  r        SSSSS   i   l  k k    e    e      *
 *             FF         r                SS  i   l  k  k   eeeee       *
 *             FF         r           S    SS  i   l  k   k  e           *
 *             FF       r            SSSSS  iii lll k   k   eeeee      *
 *                                                                         *
 ***************************************************************************
 *
 *
 *
 *
 *       ((       ))         VV    V     BBBBB                    h
 *      ((         ))        VV    V     BB   B                   h
 *     ((    cccc   ))       VV    V     BB   B  ooo   sss   cccc hhhhh
 *     ((   c       ))       VV    V     BBBBB  o   o s     c     h    h
 *     ((   c       ))        VV   V     BB   B o   o  sss  c     h    h
 *      ((  c      ))          VV V  ... BB   B o   o     s c     h    h
 *       ((  cccc ))            VV   ... BBBBB   ooo   sss   cccc hh   h
 *
 *       Kaiserstrae 49,  7000 Stuttgart 70 - Birkach,  Tel. (0711)454859
 *
 *
 *  Programmstruktur:
 *
 *  Headerdateien:  VT52.H      Bildschirmsteuerung
 *                  GalAsm.H    Makros und Def. der Variablentypen
 *                  GalGlVar.H  Deklaration der globalen Variablen
 *                  HilfFktn.H  Deklaration der Hilfsfunktionen
 *                  PrtError.H  Deklaration von PrintErrorText()
 *                  GetGal.H    Deklaration von GetGal()
 *                  Parser.H    Deklaration von Parser()
 *                  ChkArch.H   Deklaration von ChkArch()
 *                  PinInSp.H   Deklaration von PinInSpalte()
 *                  FuseMap.C   Deklaration von FuseMap()
 *                  Protokol.H  Deklaration von Protokoll()
 *
 *  C-Module:       GalAsm.C    Hauptprogramm
 *                  GalGlVar.C  Definition der globalen Variablen
 *                  HilfFktn.C  Definition der Hilfsfunktionen
 *                  PrtError.C  Definition von PrintErrorText()
 *                  GetGal.C    Definition von GetGal()
 *                  Parser.C    Definition von Parser()
 *                  ChkArch.C   Definition von ChkArch()
 *                  PinInSp.C   Definition von PinInSpalte()
 *                  FuseMap.C   Definition von FuseMap()
 *                  Protokol.C  Definition von Protokoll()
 *
 ****************************************************************************
 *
 *
 *       AA          ll         ii   t
 *      AA A          l              t
 *     AA   A  n nn   l   eeee  ii  ttt u   u n nn   gggg
 *    AA     A nn  n  l  e    e  i   t  u   u nn  n g    g
 *    AAAAAAAA n   n  l  eeeee   i   t  u   u n   n g    g
 *    AA     A n   n  l  e       i   t  u   u n   n  ggggg
 *    AA     A n   n lll  eeeee iii  tt  uuuu n   n      g
 *                                                   ggggg
 *    =====================================================
 *
 *
 * Programmaufruf:
 * --------------
 *   GalAsm <EingabeDatei> [<Ausgabedatei>] [Parameter]
 *
 *     <Eingabedatei>:  Name der Eingabedatei (MMI-Format)
 *     <Ausgabedatei>:  Name der Ausgabedatei (optional), bekommt die Extension
 *                        .Jed angehngt. Falls kein Name angegeben wird, Name
 *                        der Eingabedatei.
 *     Parameter:       Paramter, die an das Programm bergeben werden sollen.
 *     -N               Nach Programmende/-abbruch nicht auf Besttigung
 *                      durch Benutzer/-in warten. Wird bei Aufruf aus
 *                      Batch-Dateien bentigt.
 *     -F		Fix Bug in Maxon GAL Brenner Software MGP V2.0 (SYN- und
 *				AC0-Bit sind bei A-GALs vertauscht!) 			
 *
 *
 *
 * Format der Eingabedatei:
 * -----------------------
 *     ASCII-Datei nach MMI. (Siehe c't 12/89 Seite 383 und folgende)
 *     1. Zeile: GAL-Typ  (bisher untersttzt: 16V8 und 20V8)
 *
 *     2.,3. und 4. Zeile: Kommentar, wird in die Ausgabedatei
 *     bernommen. blicherweise 2. Zeile: Autor und Datum
 *                               3. Zeile: Verwendungszweck
 *                               4. Zeile: Firma,Stadt,Land usw.
 *
 *     Nun knnen die beiden Architekturkontrollbits Syn und
 *     AC0, sowie das Kontrollbyte AC1(n) angegeben werden.
 *     Form:  [Syn = X]     wobei X entweder 0 oder 1 ist.
 *            [AC0 = X]
 *            [AC1(n) = X,X,X,X,X,X,X,X]
 *     Das erste AC1(n)-Bit steuert die erste OLMC (Pin 19 bei
 *     Gal 16V8, Pin 23 bei Gal 20v8).
 *     Werden die Kontrollbits/-byte nicht oder nicht vollstn-
 *     dig angegeben, so ermittelt das Programm die fehlenden
 *     Bits/Bytes nach folgenden Prioritten:
 *          1.   Clock und/oder OE auf Matrix fhren -> Syn-Bit
 *
 *          2.a. Registered- oder Tristate-Ausgnge -> AC0 = 1
 *          2.b. Rckkopplg. der ueren OLMCs -> AC0 = 0
 *          2.c. Rckkopplg. der inneren OLMCs -> AC0 = 1
 *          2.d. AC0 = 0
 *
 *          falls AC0 == 1
 *          3.a. Ausg.n == Registered-Ausg ?          -> AC1(n)=0
 *          3.b. Ausg.n == Tristate m. Prod.'Freig. ? -> AC1(n)=1
 *          3.c. Ausg.n == Tristate m. OE-Freig. ?    -> AC1(n)=0
 *          3.d. Sonst                                -> AC1(n)=1
 *
 *          falls AC0 == 0
 *          3.a. Ausg.n == Niederohmig ?              -> AC1(n)=0
 *          3.b. Sonst                                -> AC1(n)=1
 *
 *     Folgende Zeilen: Belegung der Anschlsse mit symbolischen Namen.
 *     Die Namen mssen durch Leerzeichen getrennt werden. Sie drfen
 *     nur acht signifikante Zeichen fhren. Bei negativer Logik mu ein
 *     Nicht-Operator (/) vorangestellt werden.
 *     Die Namen werden in der Reihenfolge der zugehrigenPinnummern, be-
 *     ginnend mit Pin 1 angegeben. Masse (Gnd) und Versorgungsspannung (Vcc)
 *     mssen angegeben werden.
 *
 *     Anschlieend die logischen Gleichungen in folgender Form:
 *
 *       1. fr normale Gatter:
 *            <Name> = <Ausdruck>
 *
 *       2. fr Tri-State-Ausgnge:
 *            IF(<Produkt>)<Name>=<Ausdruck>
 *
 *       3. fr Registerausgnge (gelatchte Ausgnge):
 *            <Name> := <Ausdruck>
 *
 *       <Name>: Pin-Name, ggf. mit vorangeststelltem Nicht-Operator (/),
 *               acht signifikante Stellen.
 *
 *       <Produkt>: Ein oder mehrere Namen, verknpft mit dem Und-Operator (*).
 *
 *       IF: Bedingtes Tri-State. Wenn das in Klammern angegebene Produkt
 *           logisch wahr ist, wird der Ausgang durchgeschaltet, ansonsten
 *           ist er hochohmig (bzw. Eingang).
 *           Fehlt eine notwendige Tri-State-Bedingung, nimmt der Assembler
 *           IF(Vcc), also logisch wahr an.
 *
 *       <Ausdruck>: Folge von Produkten, logisch mit dem Oder-Operator (+)
 *                   verknpft. (Disjunktive Normalform).
 *
 *     Kommentare drfen in jeder beliebigen Zeile stehen, mssen dann aber
 *     mit einem Semikolon (;) beginnen. Der Rest der Zeile wird daraufhin
 *     ignoriert.
 *     Fhrende Leerzeichen sowie Leerzeilen werden ignoriert !
 *
 * Format der Ausgabedatei:
 * -----------------------
 *     Die ASCII-Datei enthlt die gesamte FUSE-Map der Logik-Matix,
 *     die Kontollbits der OLMCs, die Pordukttermfreigabebits,
 *     Kommentarbits und das Security-Bit im JEDEC-Format.
 *     Die Datei kann mit allen Gal-Brennern, die das JEDEC-Format lesen
 *     knnen, in ein entsprechendes Gal gebrannt werden.
 *
 *     Die 64 Kommentarbits (electronic signature) des Gals werden auf '1'
 *     gesetzt.
 *     Die Secrity-Fuse wird nicht gebrannt (G1*).
 *     Nicht bentigte Sicherungen werden nicht gebrannt (F1*).
 *     Nicht bentigte Produktzeilen werden ber die Produkttermfreigabebits
 *     gesperrt.
 *     Die einzelenen Zellengruppen werden durch Kommentare erlutert.
 *
 *
 * Hinweis:
 * -------
 *  Das Programm bentigt die ASCII-Datei GalAsm.Err zur Erklrung der
 *  Fehlermeldungen. Fr jede Fehlernummer kann ein bis zu 10 Zeilen langer
 *  Text angegeben werden. Jeder Text wird mit #F<Nummer># gekennzeichnet,
 *  wobei <Nummer> die zugeh. Fehlernummer darstellt. Die Fehlernummern mssen
 *  in der Fehlertextdatei nicht geordnet sein. nderungen sind mit jedem
 *  ASCII-Editor mglich. Sind die Texte lnger als 10 Zeilen, werden alle
 *  Zeilen nach der zehnten ignoriert. Existiert zu einer aufgetretenen
 *  Fehlernummer kein Fehlertext, so erfolgt eine entsprechende Meldung.
 *  Gibt es zu einer Fehlermeldung mehrere Eintrge, so wird der erste
 *  Eintrag der entsprechenden Fehlernummer aus der Datei angezeigt.
 *  Auch in der Fehlertextdatei sind Kommentare zugelassen. Sie mssen hier
 *  mit einem Prozentzeichen (%) beginnen. Der Zeilenrest nach einem
 *  Kommentarzeichen wird ignoriert.
 *  Falls sich die Fehlertextdatei GalAsm.Err nicht im aktuellen Verzeichnis
 *  steht, sucht das Programm in den Verzeichnissen, die in die Environment-
 *  variable GALASM eingetragen sind (z.B. Set GALASM=c:\Gal;c:\Gal\GalAsm).
 *  Vorsicht: Die Variable mu in Grobuchstaben geschrieben werden !
 *  Existiert diese Environmentvariable nicht, oder wird die Datei in keinem
 *  der angegebenen Verzeichnissen gefunden, so sucht das Programm in den
 *  Verzeichnissen, die in der Environmentvariablen PATH angegeben sind.
 *  Fhrt auch diese Suche nicht zum Erfolg, so erfolgt ein entsprechender
 *  Hinweis an den/die Benutzer/in und das Programm wird fortgesetzt.
 *
 ******************************************************************************
 *
 * Fr (sachliche und konstruktive) Kritik bin ich jederzeit dankbar.
 * Ebenso fr Hinweise auf Programmfehler.
 *
 * Programmfehler bitte mit Beschreibung der Fehlfunktion, Angabe von
 * Nummer und Erstellungsdatum der verwandten Version des Programms GalAsm,
 * sowie der Eingabedatei, die diese Fehlfunktion aufdeckte, mglichst
 * auf Disktette, an mich senden.
 *
 *
 * Volker Bosch
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "galasm.h"
#include "galglvar.h"
#include "hilffktn.h"
#include "getgal.h"
#include "parser.h"
#include "chkarch.h"
#include "pininsp.h"
#include "fusemap.h"
#include "protokol.h"
#include "prterror.h"



void fcloseall(void)
  {
   if(QuellFP != NULL) fclose(QuellFP);
   if(ZielFP  != NULL) fclose(ZielFP);
   if(ProtFP  != NULL) fclose(ProtFP);
   if(ErrFP   != NULL) fclose(ErrFP);  
  }
  
int FixMaxon = 0;  

/*------------------------------------------------------------------------
 |  Hauptprogramm.
 |  Liest Name der zu assemblierenden Datei aus Kommandozeile.
 |  ffnet Quell-, Ziel- und Fehlertextdatei und stellt den weiteren Funk-
 |  tionen Filepointer und Pufferspeicher fr Stingoperationen zur Verfgung.
 |  Ruft Unterprogramme in richtiger Reihenfolge auf.
 *--------------------------------------------------------------------------*/
int main(argc,argv)
   int argc;
   char *argv[];
 {
 char *Ptr;
 char *Ptr1;
 char *EnvPtr;
 long Fehlercode;
 int  ErrFlag;
 int i;

   ErrFlag = 0;
   printf("  GAL-Assembler GalAsm, Ver. %s, vom %s (c) by V.Bosch\n\n",
           VersionNr,ErstDatum);
   if( ! (Buffer=malloc(ZEILENLAENGE+4))) {  /* Zeilenspeicher anfordern */
     fprintf(stderr,"*** Fehler: zu wenig freier Arbeitsspeicher ***\n");
     exit(1);
   }
   if (argc < 2)  {
    fprintf(stderr,
	    "\nProgrammaufruf: GalAsm  <EQN-Datei> [<Name Ausgabedatei>] [Parameter]\n");
    fprintf(stderr,"\tParameter: -%c: Fix Bug in MGP-Software Ver. 2.0 (AC0 und SYN bit vertauschen)\n",
    			FIX_MAXON);
     exit(2);
   }
   if( ! (QuellFP = fopen(argv[1],"r"))) {
     strcpy(Buffer,argv[1]);      /* Name der Eingabedatei kopieren... */
     Ptr = Buffer;
     while((*Ptr) && (*Ptr !='.')) Ptr++;   /* Punkt suchen */
     if(!(*Ptr)) {                          /* Punkt nicht gefunden */
       *Ptr = '.';                          /* Punkt eintragen */
       Ptr++;                               /* Zeiger auf nchstes Zeichen */
       strcpy(Ptr,DEFAULT_EXT);               /* Defaultextension anhngen */
       if( ! (QuellFP = fopen(Buffer,"r"))) {
         fprintf(stderr,"*** Fehler: Datei %s nicht zu ffnen ***\n",Buffer);
         exit(3);
       }
     }
     else {
       fprintf(stderr, "*** Fehler: Datei %s nicht zu ffnen ***\n",argv[1]);
       exit(3);
     }
   }
   for(i = 0; i < UES_LEN; i++) {
     UserElSig[i] = '\0';             /* String nullen */
   }
   Ptr = Ptr1 = argv[1];    /* Den Namen des Files suchen */
   while(*Ptr) {
     if(*Ptr++ == '/') Ptr1 = Ptr;
   }
   if(*Ptr1) {
     strncpy(UserElSig,Ptr1,UES_LEN); /* 8 Stellen des Filenamens kopieren */
     UserElSig[UES_LEN] = '\0';       /* ...und mit 0-Byte abschlieen */
     Ptr = UserElSig;
     while(*Ptr) {                    /* Extension suchen, ...*/
       if(*Ptr == '.') {
         *Ptr++ = '\0';               /* String abschneiden... */
         while(*Ptr) *Ptr++ = '\0';   /* und Stringende nullen */
       }
       else Ptr++;
     }
   }
   if( !(ErrFP = fopen(ErrDatNam,"r"))) {       /* Fehlertextdatei ffnen */
     /*-- Fehlertextdat. n. gefunden -> im Environment nachsehen --------*/
     if((EnvPtr = getenv(GALASM_ENV_NAM))) {   /* Environmentvariable holen */
       /*-- Alle durch Doppelpunkt getrennten Eintrge durchprobieren ------*/
       do {
         stccpy(Buffer,EnvPtr,ZEILENLAENGE+2);      /* Env'string -> Puffer */
         Ptr = Buffer;
         while(*Ptr) {
           if ((*Ptr) == ':')  {      /* 1. Semikolon suchen */
             *Ptr   = '\0';           /* String abschneiden */
             break;                   /* while-Schleife verlassen */
           }
           Ptr++;
         }
         *Ptr++ = '/';
         *Ptr = '\0';
         strncat(Buffer,ErrDatNam,(ZEILENLAENGE+3-strlen(ErrDatNam)));
         if((ErrFP = fopen(Buffer,"r"))) {   /* Fehlertextdatei ffnen */
            break;                           /* erfolgreich -> Schleifenende */
         }
         while((*EnvPtr) &&         /* Env'string hinter nchstes Semik. */
               ((*EnvPtr) != ':') )  EnvPtr++;
         if(*EnvPtr) EnvPtr++;
       } while(*EnvPtr);            /* Bis Env. String abgearbeitet */
     }
     if(!ErrFP) {    /* Fehlertextdatei noch nicht geffnet */
       if((EnvPtr = getenv(PATH_ENV_NAM))) {  /* Environmentvar. PATH holen */
         /*-- Alle durch Doppelpunkt getrnnten Eintrge durchprobieren ------*/
         do {
           stccpy(Buffer,EnvPtr,ZEILENLAENGE+3);
           Ptr = Buffer;
           while(*Ptr) {                /* 1. Semikolon suchen */
             if ((*Ptr) == ':')  {      
               *Ptr   = '\0';           /* String abschneiden */
               break;
             }
             Ptr++;
           }
           *Ptr++ = '/';
           *Ptr = '\0';
           strncat(Buffer,ErrDatNam,ZEILENLAENGE+3-strlen(ErrDatNam));
           if((ErrFP = fopen(Buffer,"r"))) {   /* Fehlertextdatei ffnen */
             break;                          /* erfolgreich -> Schleifenende */
           }
           while((*EnvPtr) &&              /* Nchsten Eintrag suchen */
                 ((*EnvPtr) != ':') )  {
              EnvPtr++;
           }
           if(*EnvPtr) EnvPtr++;
         } while(*EnvPtr);        /* bis Env.string abgearbeitet */
       }
     }
     if(!ErrFP) {
       fprintf(stderr,"*** Warnung: Fehler-Text-Datei %s nicht zu ffnen ***\n",
               ErrDatNam);
     }
   gotoxy(0,6);
   }

   printf("Bearbeite Datei %s -> ",argv[1]);         /* Ausgabe Zeile 6 */
   if(argc < 3) {     /* kein Zieldateiname angegeben */
     stccpy(Buffer,argv[1],MAX_PATH_LEN-1);   /* Quelldateiname kopieren */
   } else {
     if(*argv[2] != PAR_KENNG)                 /* Kein Parameter bergeben */
       stccpy(Buffer,argv[2],MAX_PATH_LEN-1);   /* Zieldateiname kopieren */
     else
       stccpy(Buffer,argv[1],MAX_PATH_LEN-1);   /* Quelldateiname kopieren */
   }
   Ptr = Buffer;
   while((*Ptr && (*Ptr !='.'))) Ptr++;      /* '.' oder Stringende suchen */
   if(! *Ptr) {    /* '.' nicht gefunden, d.h. *Ptr == NULL-Byte */
     *Ptr = '.';        /* '.' anfgen */
   }
   Ptr++;
   strcpy(Ptr,ZIEL_EXT);                        /* Extension anhngen */
   printf("%s\n",Buffer);                       /* Ausgabe Zeile 6 beendet */
   if( ! (ZielFP = fopen(Buffer,"w+"))) {
     fprintf(stderr,"*** Fehler: Zieldatei %s nicht zu ffnen ***\n",Buffer);
     fcloseall();
     exit(4);
   }
   Ptr = stpchr(Buffer,'.');
   Ptr++;
   strcpy(Ptr,PROT_EXT);             /* Extension anhngen */
   if( !(ProtFP = fopen(Buffer,"w"))) {
     fprintf(stderr, "*** Fehler: Datei %s nicht zu ffnen ***\n",Buffer);
     fcloseall();
     exit(5);
   }
   if(argc > 2) {   /* mehr als 2 Argumente bergeben */
     i = 2;
     do {
       if(*argv[i] == PAR_KENNG) {
        switch (toupper(*(argv[i]+1))) {
           case WARTEN: {
           } break;
           case FIX_MAXON: {
             FixMaxon = 1;
           } break;
           default:  {
             fprintf(stderr, "*** Fehler: Unbekannter Paramter %s ***\n",argv[i]);
             exit(6);
           }
         }
       }
     i++;
     } while(i < argc);
   }
   if(FixMaxon != 0) {
      printf("Fix fuer MAXon Galprog: AC0- und Syn-Bit vertauscht!\n");
   }   
   if((Fehlercode = GetGal()) != 0) {
     ErrFlag = 1;
     gotoxy(0,MSG_ZEILE);
     end_screen();
     fprintf(stderr, "*** Fehler %ld bei Analyse der Signalnamen Zeilennummer %d ***\n",
     Fehlercode,QuellZeilNr);
     PrintErrorTxt(Fehlercode);
     fcloseall();
     exit(7);
   }
   if((Fehlercode = Parser()) != 0) {
     ErrFlag = 1;
     gotoxy(0,MSG_ZEILE);
     end_screen();
     fprintf(stderr,"*** Fehler %ld bei Parser-Duchlauf in Zeilenr: %d ***\n",
     Fehlercode,QuellZeilNr);
     PrintErrorTxt(Fehlercode);
     fcloseall();
     exit(8);
   }
   if((Fehlercode = ChkArch()) != 0) {
     ErrFlag = 1;
     gotoxy(0,MSG_ZEILE);
     end_screen();
     fprintf(stderr,"\n\n\n*** Fehler %ld bei Kontrolle der Architekturkontrollbits ***\n",
             Fehlercode);
     PrintErrorTxt(Fehlercode);
   }
   if((Fehlercode = PinInSpalte()) != 0) {
     ErrFlag = 1;
     gotoxy(0,MSG_ZEILE);
     end_screen();
     fprintf(stderr,"*** Fehler %ld bei Aufteilung der Pin auf Spaltennummern ***\n",
             Fehlercode);
     PrintErrorTxt(Fehlercode);
     fcloseall();
     exit(10);
   }
   if((Fehlercode = FuseMap()) != 0) {
     ErrFlag = 1;
     gotoxy(0,MSG_ZEILE);
     end_screen();
     fprintf(stderr,"*** Fehler %ld bei Anlegen der JEDEC-Datei ***\n",
             Fehlercode);
     PrintErrorTxt(Fehlercode);
     fcloseall();
     exit(11);
   }
   if((Fehlercode = Protokoll(argv[1])) != 0) {
     ErrFlag = 1;
     gotoxy(0,MSG_ZEILE);
     end_screen();
     fprintf(stderr,"*** Fehler %ld bei Anlegen der Protokoll-Datei ***\n",
             Fehlercode);
     PrintErrorTxt(Fehlercode);
     fcloseall();
     exit(12);
   }
   if(! ErrFlag) {
     gotoxy(0,MSG_ZEILE);
     end_screen();
     printf("Assemblierung erfolgreich beendet !\n");
     fcloseall();
     return(KEIN_FEHLER);
   }
   gotoxy(0,END_ZEILE);
   end_screen();
   fprintf(stderr,"*** Fehler: Assemblierung mit Fehlern beendet !***\n");
   fcloseall();
   return(1);
 }   /* Ende von main() */
