/******************************************************************************
 * Headerdatei HilfFktn.H
 *
 *  Deklariert Funktionen, welche von den einzelnen Programmteilen des
 *  Programms bentigt werden. Fr GalAsm, den Logic-Compiler / -Assembler
 *  zur Erstellung einer JEDEC-Datei fr die GAL-Programmierung.
 *
 *  Version               0.1
 *  Erstellt am           11.08.1990
 *  Letzte nderung am    11.08.1990
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *****************************************************************************/



/*----------------------------------------------------------------------
 | berliest die Leerzeichen, auf die der Zeiger ZeilPtr zeigt.
 | wenn Zeilenende (NULL-Byte) oder Kommentar (;) erreicht, wird
 | eine neue Zeile gelesen. Rckgabewert: Zeiger auf erstes Zeichen, das
 | kein Leerzeichen mehr ist.
 | Bei Fehler (unerwartetes Dateiende) wird ein NULL-Zeiger zurckgegeben
 *-------------------------------------------------------------------------*/
extern char *LeerzUeberl(char *);


/*---------------------------------------------------------------------------
 | Funktion SingalNegativ() stellt fest, ob das Signal, auf dessen Namen
 | der ersrte Zeiger zeigt einen oder mehere Not-Operatoren (/) fhrt.
 | Der erste Not-Operator setzt das Negativflag, das per reverence als zweiter
 | Zeiger bergeben werden mu, alle weiteren toggeln das Flag. Wird kein
 | Not-Operator gefunden, so wird das Naegativflag gelscht.
 | Rckgabewert: Neue Stellung des ersten Zeigers. Im Fehlerfall NULL.
 *---------------------------------------------------------------------------*/
extern char *SignalNeg(char *, char *);


/*-------------------------------------------------------------------
 |  Funktion liest eine Zeile aus der Quelldatei ein und gibt einen Zeiger
 |  auf das erste Zeichen, das kein Leerzeichen ist, zurck.
 |  CR und LF am Zeilenende werden durch NULL-Bytes ersetzt.
 |  Leer- und reine Kommentarzeilen werden berlesen.
 |  Rckabewert: NULL-Zeiger, falls IO-Fehler (wie z.B. Datei leer).
 *---------------------------------------------------------------------*/
extern char *LeseZeile(void);


/*-------------------------------------------------------------------------
 | Funktion kopiert aus der Zeichenkette, auf der erste Zeiger zeigt, den
 | ersten Signalnamen in die Zeichenkette, auf die der zweite Zeiger zeigt.
 | Bercksichtigt werden nur die acht ersten Stellen.
 | Bei Erreichen des ersten nicht alphanummerischen Zeichens wird der
 | Kopiervorgang beendet und die herauskopierte Zeichenkette mit einem NULL-
 | Byte abgeschlossen.
 | Rckgabe: Neue Position des ersten Zeigers, auf das erste folgende
 | nicht alphanummerische Zeichen.
 *---------------------------------------------------------------------------*/
extern char *GetSigName(char *,char *);


/*----------------------------------------------------------------------
 |  Funktion GetEingPinNr(SigName)
 |  Lifert die Nummer des Eingangspins an dem das Signal mit dem Namen
 |  SigName anliegt und setzt ggf. die Flags ClkinMatrix (Clockeingang
 |  mu auf Logikmatrix gefhrt werden) und OEinMatrix (OE-Signal mu
 |  mu auf Logikmatrix gefhrt werden).
 |  Im Fehlerfall Rckgabewert: 0.
 *-----------------------------------------------------------------------*/
extern int GetEingPinNr(char *);


/*----------------------------------------------------------------------
 |  Funktion GetAusgPinNr(SigName)
 |  Lifert die Nummer des Ausgangspins an dem das Signal mit dem Namen
 |  SigName anliegt. auerdem wird fr den zugeh. Ausgang das Flag
 |  Rueckkop gesetzt.
 |  Im Fehlerfall Rckgabewert 0.
 *-------------------------------------------------------------------------*/
extern int GetAusgPinNr(char *);

/*--------------------------------------------------------------------------
 | Funktion ProdTermFrei() prft ob in dem dem bergebenen Ausgang (0 bis
 | AUSGAENGE-1) noch ein Produktterm frei ist und kopiert ggf. den 0. Pro-
 | duktterm an die freie Stelle. Der 0. Term wird gelscht.
 | Rckgabewert: <0 im Fehlerfall, sonst 0.
 *---------------------------------------------------------------------------*/
extern int ProdTermFrei(int);
