/******************************************************************************
 *  Datei PrtError.C
 *  Enthlt die Funktion PrintErrorTxt() fr GalAsm, den Logic-Compiler / -
 *  Assembler zur Erstellung einer JEDEC-Datei fr die GAL-Programmierung.
 *
 *  Version               1.1
 *  Erstellt am           10.12.1990
 *  Letzte nderung am    22.12.1990
 *  Verwendeter Compiler: Lattice C for Atari ST, Ver 3.04
 *
 *  (c) Volker Bosch, Kaiserstrae 49, 7000 Stuttgart 70, (0711)454859
 *
 *  Bentigte Headerdateien:  GalAsm.H     Konstanten und Def. der Var.'typen
 *                            GalGlVar.H   Deklaration der globalen Variablen
 *
 *  Bentigte C-Module:       GalGlVar.C
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>


#include "galasm.h"
#include "galglvar.h"


/*------- Deklaration der internen Hilfsfunktion ----------*/
char *LeseErrZeile(void);


/*------- Globale Variablen -------------*/
int ErrDatZeile = 0;


/*-------------------------------------------------------------------
 |  Funktion liest eine Zeile aus der Fehlertextdatei ein und gibt einen
 |  Zeiger auf das erste Zeichen, das kein Leerzeichen ist, zurck.
 |  CR und LF am Zeilenende werden durch NULL-Bytes ersetzt.
 |  Leer- und reine Kommentarzeilen werden berlesen.
 |  Rckabewert: NULL-Zeiger, falls IO-Fehler (wie z.B. Datei leer).
 *---------------------------------------------------------------------*/
char *LeseErrZeile()
{
  char *Ptr;
  char *HPtr;    /* Hilfszeiger */

  do {
    ErrDatZeile++;
    if( ! (Ptr = fgets(Buffer,ZEILENLAENGE,ErrFP))) return(NULL);
    HPtr = Ptr;
    while(*HPtr) {     /* CR und LF im String durch NULL-Bytes ersetzen */
      if (*HPtr == CR) {
        *HPtr = '\0';
        break;
      }
      if (*HPtr == LF)  {
        *HPtr = '\0';
        break;
      }
      if (*HPtr == ERR_KOMMENTAR) {
        *HPtr = '\0';
        break;
      }
      HPtr++;
    }
    while((*Ptr) && (*Ptr==' '))
       Ptr++;           /* Leerzeichen berlesen */
  } while (! *Ptr);                   /* wenn Zeile leer, neue Zeile lesen */
  return(Ptr);
}

/*-------------------------------------------------------------------------
 | Funktion PrintErrorTxt(FehlerNr)
 | Zeigt den zur Fehlernummer <FehlerNr> gehrenden Text aus der Fehlertext-
 | datei <ErrDatNam> an, falls die Datei in der Funktion main() geffnet
 | wurde und der Eintrag fr die entsprechende Fehlernummer existiert.
 | Rckgabewert: Keiner.
 *--------------------------------------------------------------------------*/
void PrintErrorTxt(FehlerNr)
long FehlerNr;
{
  long ErrTextNr;
  int LfdErrZeile;
  char *Zeile;

  if(! (ErrFP)) return;    /* Fehlertextdatei nicht geffnet ! */
  ErrTextNr = 0;
  Zeile = NULL;
  while((ErrTextNr >= 0) &&
        (ErrTextNr != FehlerNr) ) {
     do {
        Zeile = LeseErrZeile();           /* erreicht oder Fehlernummer */
     if(! Zeile) {
       fprintf(stderr,"*** Fehlercode %ld nicht in Datei %s gefunden ***\n",
               FehlerNr,ErrDatNam);
       return;
     }
     } while((Zeile) &&                   /* Solange Zeilen auf der Fehler- */
             (*Zeile != ERR_NUM_KENNG) && /* textdatei lesen, bis Dateiende */
             (*(Zeile+1) != 'F')          );
     Zeile+=2;
     if(isdigit(*Zeile)) {
        ErrTextNr =  strtol(Zeile, (char **)NULL, 10);
     }
  }  /* while((ErrTextNr >= 0)&&.. */
  if(! ErrTextNr) {
    fprintf(stderr,"*** Fehlernummer %ld nicht in Datei %s enthalten ***\n",
             FehlerNr,ErrDatNam);
    return;
  }
  while((*Zeile) &&                    /* Ende der Fehlernummer suchen */
        (*Zeile != ERR_NUM_KENNG)  )   Zeile++;
    if(! *Zeile) {
      fprintf(stderr,"*** Falscher Eintrag in Datei %s, Fehlernr. %ld, Zeile %d ***\n",
              ErrDatNam,ErrTextNr,ErrDatZeile);
      return;
    }
    Zeile++;             /* Zeiger auf nchstes Zeichen */
    while((*Zeile) &&
          ((*Zeile ==' ')||
          (! (isprint(*Zeile))) ) )  Zeile++;
    LfdErrZeile = 1;     /* Zhler fr Zeilennummer initialisieren */
    if(! *Zeile) {                /* Zeilenende erreicht =>   */
      Zeile = LeseErrZeile();     /* nchste Zeile lesen */
    }
    while((Zeile)                     &&      /* solange nicht Dateiende oder*/
          (*Zeile != ERR_NUM_KENNG)   &&      /* nchster Fehlertext oder */
          (LfdErrZeile <= ANZ_ERR_ZEILEN)  ) { /* mehr als 10 Zeilen gelesen */
      fprintf(stderr,"%s\n",Zeile);    /* Textzeile anzeigen */
      LfdErrZeile++;           /* Zhler fr Zeilennummer inkrementieren */
      Zeile = LeseErrZeile();  /* nchste Zeile Lesen */
    } /* while((Zeile) && .. */
    return;
}

